@@ -1,8 +1,8 @@
 use super::*;
 
-/// An alias, e.g. `name := target`
+/// An alias, e.g. `alias name := target`
 #[derive(Debug, PartialEq, Clone, Serialize)]
-pub(crate) struct Alias<'src, T = Rc<Recipe<'src>>> {
+pub(crate) struct Alias<'src, T = Arc<Recipe<'src>>> {
   pub(crate) attributes: AttributeSet<'src>,
   pub(crate) name: Name<'src>,
   #[serde(
@@ -13,8 +13,8 @@ pub(crate) struct Alias<'src, T = Rc<Recipe<'src>>> {
 }
 
 impl<'src> Alias<'src, Namepath<'src>> {
-  pub(crate) fn resolve(self, target: Rc<Recipe<'src>>) -> Alias<'src> {
-    assert!(self.target.last().lexeme() == target.namepath.last().lexeme());
+  pub(crate) fn resolve(self, target: Arc<Recipe<'src>>) -> Alias<'src> {
+    assert!(self.target.last().lexeme() == target.name());
 
     Alias {
       attributes: self.attributes,
@@ -25,8 +25,9 @@ impl<'src> Alias<'src, Namepath<'src>> {
 }
 
 impl Alias<'_> {
-  pub(crate) fn is_private(&self) -> bool {
-    self.name.lexeme().starts_with('_') || self.attributes.contains(AttributeDiscriminant::Private)
+  pub(crate) fn is_public(&self) -> bool {
+    !self.name.lexeme().starts_with('_')
+      && !self.attributes.contains(AttributeDiscriminant::Private)
   }
 }
 
