@@ -2,12 +2,16 @@ use super::*;
 
 #[derive(Debug, PartialEq)]
 pub(crate) enum CompileErrorKind<'src> {
+  ArgumentPatternRegex {
+    source: regex::Error,
+  },
   AttributeArgumentCountMismatch {
     attribute: &'src str,
     found: usize,
     min: usize,
     max: usize,
   },
+  AttributePositionalFollowsKeyword,
   BacktickShebang,
   CircularRecipeDependency {
     recipe: &'src str,
@@ -23,6 +27,10 @@ pub(crate) enum CompileErrorKind<'src> {
     min: usize,
     max: usize,
   },
+  DuplicateArgAttribute {
+    arg: String,
+    first: usize,
+  },
   DuplicateAttribute {
     attribute: &'src str,
     first: usize,
@@ -103,12 +111,12 @@ pub(crate) enum CompileErrorKind<'src> {
   RequiredParameterFollowsDefaultParameter {
     parameter: &'src str,
   },
-  ShebangAndScriptAttribute {
-    recipe: &'src str,
-  },
   ShellExpansion {
     err: shellexpand::LookupError<env::VarError>,
   },
+  UndefinedArgAttribute {
+    argument: String,
+  },
   UndefinedVariable {
     variable: &'src str,
   },
@@ -146,6 +154,10 @@ pub(crate) enum CompileErrorKind<'src> {
   UnknownAttribute {
     attribute: &'src str,
   },
+  UnknownAttributeKeyword {
+    attribute: &'src str,
+    keyword: &'src str,
+  },
   UnknownDependency {
     recipe: &'src str,
     unknown: Namepath<'src>,
