@@ -1,14 +1,17 @@
-#[derive(PartialEq, Eq, Debug, Copy, Clone)]
+use super::*;
+
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub(crate) enum Delimiter {
   Brace,
   Bracket,
+  FormatString(StringKind),
   Paren,
 }
 
 impl Delimiter {
   pub(crate) fn open(self) -> char {
     match self {
-      Self::Brace => '{',
+      Self::Brace | Self::FormatString(_) => '{',
       Self::Bracket => '[',
       Self::Paren => '(',
     }
@@ -16,7 +19,7 @@ impl Delimiter {
 
   pub(crate) fn close(self) -> char {
     match self {
-      Self::Brace => '}',
+      Self::Brace | Self::FormatString(_) => '}',
       Self::Bracket => ']',
       Self::Paren => ')',
     }
