@@ -26,6 +26,9 @@ pub(crate) enum TokenKind {
   Equals,
   EqualsEquals,
   EqualsTilde,
+  FormatStringContinue,
+  FormatStringEnd,
+  FormatStringStart,
   Identifier,
   Indent,
   InterpolationEnd,
@@ -72,6 +75,7 @@ impl Display for TokenKind {
         Equals => "'='",
         EqualsEquals => "'=='",
         EqualsTilde => "'=~'",
+        FormatStringContinue | FormatStringEnd | FormatStringStart => "format string",
         Identifier => "identifier",
         Indent => "indent",
         InterpolationEnd => "'}}'",
