@@ -5,6 +5,12 @@ fn constants_are_defined() {
   assert_eval_eq("HEX", "0123456789abcdef");
 }
 
+#[test]
+fn constants_can_have_different_values_on_windows() {
+  assert_eval_eq("PATH_SEP", if cfg!(windows) { "\\" } else { "/" });
+  assert_eval_eq("PATH_VAR_SEP", if cfg!(windows) { ";" } else { ":" });
+}
+
 #[test]
 fn constants_are_defined_in_recipe_bodies() {
   Test::new()
